/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ev;

import ev.cc.ParentSelector;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.logging.Logger;
import org.reflections.Reflections;

/**
 *
 * @author sigved
 */
public class Evolution<P extends Pheno, G extends Geno> {
    Set<P> pop = new HashSet<P>();
    List<PopulationFunction> functions = new ArrayList<PopulationFunction>();
    private int stage = -1;
    public synchronized Set<P> getPop(){
        return pop;
    }
    public void addPheno(P p){
        pop.add(p);
    }
    public synchronized boolean evolve(){
        stage = -1;
        for(PopulationFunction f : functions){
            stage++;
            log.info("running function " + f + " pop size: " + pop.size());
            pop = f.runOnPopulation(pop);
        }
        return true;
    }
    public List<PopulationFunction> getFunctions(){
        return functions;
    }
    public int getStage(){
        return stage;
    }
    public static Logger log = Logger.getLogger("ev");
    static{
        //log.setLevel(Level.OFF);
        log.info("available ParentSelectors: " + ParentSelector.getParentSelectors());
        log.info("available Population Function: " + getPopulationFunction());
    }
    public boolean evolve(int iterations){
        log.info("starting " + iterations + " iterations long evolution");
        for(int i = 0; i < iterations; i++)
            if(!evolve()){
                log.warning("failed to complete evolution iterationnumber " + i
                         + " stopping");
                return false;
            }
        log.info("completed " + iterations + " iterations long evolution");
        return true;
    }
    
    public static Set<Class<? extends PopulationFunction>> getPopulationFunction(){
        Reflections ref = new Reflections("ev");
        
        return ref.getSubTypesOf(PopulationFunction.class);
    }
    
    public static Set<Class<? extends ParentSelector>> getParentSelectionFunction(){
        Reflections ref = new Reflections("");
        
        return ref.getSubTypesOf(ParentSelector.class);
    }

    public double getPopulationSize() {
        return this.pop.size();
    }

    public double getBestFitness() {
        
        return getBestPheno().getFitness();
    }

    public double getMedianFitness() {
        double sum = 0;
        for(Pheno p : pop)sum += p.getFitness();
        return sum/this.getPopulationSize();
    }

    public double getWorstFitness() {
        TreeSet<Pheno> ts = new TreeSet<Pheno>(new Comparator <Pheno>(){
            @Override
            public int compare(Pheno arg0, Pheno arg1) {
                if(arg0.getFitness() >= arg1.getFitness())return 1;
                else return -1;
            }
        });
        ts.addAll(pop);
        return ts.first().getFitness();
    }

    public P getBestPheno() {
        TreeSet<P> ts = new TreeSet<P>(new Comparator <Pheno>(){
            @Override
            public int compare(Pheno arg0, Pheno arg1) {
                if(arg0.getFitness() >= arg1.getFitness())return 1;
                else return -1;
            }
        });
        ts.addAll(pop);
        return ts.last();
    }
    
    public TreeSet<P> sortedByFitness(){
        TreeSet<P> phenoTree = new TreeSet<P>(new Comparator<Pheno>(){

            @Override
            public int compare(Pheno arg0, Pheno arg1) {
                if(arg0.getFitness() > arg1.getFitness())return 1;
                else return -1;
            }

        });
        phenoTree.addAll(this.getPop());
        return phenoTree;
    }
}
