
package ev.ui.servlet;

import ev.Evolution;
import ev.Pheno;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.TreeSet;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.vecmath.Vector2d;
import org.eclipse.jetty.server.Handler;
import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.handler.AbstractHandler;
import org.eclipse.jetty.server.handler.ContextHandler;
import org.eclipse.jetty.server.handler.ContextHandlerCollection;
import swingplotter.Graph;
import swingplotter.PlotterPanel;


/**
 *
 * @author sigved
 */
public class EVServer{
    private final Server server;
    protected final Evolution ev;
    private final Graph graphPopSize;
    private final Graph graphBestFitness;
    private final Graph graphMedianFitness;
    private final Graph graphWorstFitnes;
    protected final PlotterPanel graphPane = new PlotterPanel();
    protected TreeSet<Pheno> ts;
    protected final ContextHandlerCollection hcc;
    @SuppressWarnings("LeakingThisInConstructor")
    public EVServer(int port, Evolution ev) throws Exception{
        this.ev = ev;
        
        ArrayList<Graph> graphs = new ArrayList<Graph>();
        graphs.add(this.graphPopSize = new Graph(Color.blue));
        graphs.add(this.graphBestFitness = new Graph(Color.yellow));
        graphs.add(this.graphMedianFitness = new Graph(Color.red));
        graphs.add(this.graphWorstFitnes = new Graph(Color.black));
        graphPane.setGraphs(graphs);
        
        graphPane.setPos(new Vector2d(-10.0d, 0.0d));
        graphPane.setScale(new Vector2d(0.2d, 0.001d));
        
        server = new Server(port);
        
        hcc = new ContextHandlerCollection();
        
        
        ContextHandler chPlot = new ContextHandler("/plot");
        chPlot.setHandler(new PlotterHandler());
        
        ContextHandler chOverview = new ContextHandler("/overview");
        chOverview.setHandler(new OverviewHandler());
        
        ContextHandler chSettings = new ContextHandler("/settings");
        chSettings.setHandler(new EvolutionSettings.SettingsHandler(ev));
        
        ContextHandler chGANDP = new ContextHandler("/graphandpop");
        chGANDP.setHandler(new GraphAndPopulationHandler());
        
        hcc.setHandlers(new Handler[]{chOverview, chPlot, chSettings, chGANDP});
        server.setHandler(hcc);
        server.start();
//        server.join();
    }
    
    public void restart() throws Exception{
        server.stop();
        server.setHandler(hcc);
        server.start();
    }

    private class PlotterHandler extends AbstractHandler{
        @Override
        public void handle(String string, Request rqst, HttpServletRequest hsr, HttpServletResponse res) throws IOException, ServletException {
            res.setContentType("image/jpeg");
            res.setStatus(HttpServletResponse.SC_OK);
            rqst.setHandled(true);
            try{
                BufferedImage bufferedImage = new BufferedImage(800, 1000, BufferedImage.TYPE_INT_RGB);
                Graphics g = bufferedImage.getGraphics();
                graphPane.setSize(800, 1000);
                synchronized(graphPane){graphPane.paint(g);}
                g.dispose();
                //Write the image as a jpg
                ImageIO.write(bufferedImage, "jpg", res.getOutputStream());
            }catch (IOException ioe){}
        }
    }
    
    private class OverviewHandler extends AbstractHandler{
        HttpServletResponse res;
        @Override
        public void handle(String string, Request rqst, HttpServletRequest hsr, HttpServletResponse res) throws IOException{
            this.res = res;
            res.setContentType("text/html");
            res.setStatus(HttpServletResponse.SC_OK);
            rqst.setHandled(true);
//            if(ts == null){
//                print("NOTHING HERE YET, WAIT...");
//                return;
//            }
            print("<html><head>");
//            print("<meta http-equiv='refresh' content='5'>");
            print("</head><body><br>");
            print(EvolutionSettings.getSettingsPageFor(ev));
            print("<iframe src='/graphandpop' style='width:900;height:900'></iframe>");
            print("</body></html>");
        }
        
        public void print(String s) throws IOException{
            res.getWriter().println(s);
        }
    }
    
    private class GraphAndPopulationHandler extends AbstractHandler{
        HttpServletResponse res;
        @Override
        public void handle(String string, Request rqst, HttpServletRequest hsr, HttpServletResponse res) throws IOException{
            this.res = res;
            res.setContentType("text/html");
            res.setStatus(HttpServletResponse.SC_OK);
            rqst.setHandled(true);
            if(ts == null){
                print("NOTHING HERE YET, WAIT and refresh");
                return;
            }
            print("<html><head><tittle>iteration " + i + "</tittle>");
            print("<meta http-equiv='refresh' content='5'>");
            print("</head><body><br>");
            print("<div id='overwiev' style='float:center'>");
            print("<img src='/plot'>");
            print("<ol>");
            synchronized(graphPane){
                for(Pheno p : ts.descendingSet()){
                    print("<li>"+p.toString()+"</li>");
                }
            }
            print("</ol>");
            print("</div></body></html>");
        }
        
        public void print(String s) throws IOException{
            res.getWriter().println(s);
        }
    }
    
    protected int i = 0;
    public void run(){
        System.out.println("Starting EVServer");
////catch initpop:
//        try {
//            runGraphs();
//        } catch (FileNotFoundException ex) {
//            Logger.getLogger(EVServer.class.getName()).log(Level.SEVERE, null, ex);
//        }
//                    try {
//                Thread.sleep(1000000);
//            } catch (InterruptedException ex) {
//                Logger.getLogger(EVServer.class.getName()).log(Level.SEVERE, null, ex);
//            }
        for(i = 0; i < 250000; i++){
            ev.evolve();
            synchronized(graphPane){
                try {
                    runGraphs();
                } catch (FileNotFoundException ex) {
                    Logger.getLogger(EVServer.class.getName()).log(Level.SEVERE, null, ex);
                }
            }

        }
    }
    
    public void runGraphs() throws FileNotFoundException{
        graphPane.setPos(new Vector2d(Math.max(-10.0d, i-150d), 0.0d));
        this.graphPopSize.addPoint(i, this.ev.getPopulationSize());
        this.graphBestFitness.addPoint(i, this.ev.getBestFitness());
        this.graphMedianFitness.addPoint(i, this.ev.getMedianFitness());
        this.graphWorstFitnes.addPoint(i, this.ev.getWorstFitness());
        ts = ev.sortedByFitness();
//        this.graphBestFitness.toFile("EVServer");
    }
}
