
package ev.ui.servlet;

import ev.Evolution;
import ev.PopulationFunction;
import java.io.IOException;
import java.lang.reflect.Field;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.handler.AbstractHandler;

/**
 *
 * @author sigved
 */
public class EvolutionSettings {

    static String getSettingsPageFor(Evolution ev) {
        String ret = "";
        ret += "<div id = 'settingsContainer' style='width:100%'>";
        ret += "<div id = 'header'>settings:</div>";
        
        ret += "<div id='menu' style='float:left;width:'25%'>menu:";
        List<PopulationFunction> pfs = ev.getFunctions();
        for(PopulationFunction pf : pfs){
            ret += "<br><a href=/settings/'" + pf.toString() + "' target='detailFrame'>" + pf.toString() + "</a>";
        }
        ret += "</div>";
        ret += "<div id='item' style='float:left;width:'75%'><iframe name='detailFrame' width='600'></iframe></div>";
        ret += "</div>";
        return ret;
    }
    
    public static class SettingsHandler extends AbstractHandler{
        private final Evolution ev;
        public SettingsHandler(Evolution ev){
            this.ev = ev;
        }
        @Override
        public void handle(String string, Request rqst, HttpServletRequest hsr, HttpServletResponse res) throws IOException, ServletException {
            res.setContentType("text/html");
            res.setStatus(HttpServletResponse.SC_OK);
            rqst.setHandled(true);
            if(rqst.getParameterMap().size() > 0){
                handleParameterChange(rqst.getParameterMap(), rqst.getUri().toString());
            }
            String ret = "";
            ret += createGenericSettingsForm(rqst.getUri().toString());
            res.getWriter().println(ret);
        }
        public String createGenericSettingsForm(String uri){
            String ret = "";
            List<PopulationFunction> pfs = ev.getFunctions();
            for(PopulationFunction pf : pfs){
                if(uri.contains(pf.toString())){
                    return createGSForm(pf);
                }
            }
            return "not found";
        }
        public String createGSForm(PopulationFunction pf){
            String ret = "<form name='" + pf.toString() +  "' action='" + pf.toString() + "' method='POST'>";
            for(final Field f : pf.getClass().getFields()){
                try {
                    ret += f.getName() + ": " + "<input type='text' name='" + f.getName() + "' value='" + f.getDouble(pf) + "'><br>";
                } catch (IllegalArgumentException ex) {
                    Logger.getLogger(EvolutionSettings.class.getName()).log(Level.SEVERE, null, ex);
                } catch (IllegalAccessException ex) {
                    Logger.getLogger(EvolutionSettings.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
            if(pf instanceof AdditionalForm){
                ret += ((AdditionalForm)pf).getAdditionalForm();
            }
            ret += "<input type='submit' value='change'>";
            ret += "</form>";
            return ret;
        }

        private void handleParameterChange(Map<String, String[]> parameterMap, String uri) {
            List<PopulationFunction> pfs = ev.getFunctions();
            for(PopulationFunction pf : pfs){
                if(uri.contains(pf.toString())){
                    for(Field f : pf.getClass().getFields()){
                        if(parameterMap.containsKey(f.getName())){
                            try {
                                if(f.getType().equals(double.class))
                                    f.setDouble(pf, Double.parseDouble(parameterMap.get(f.getName())[0]));
                                else if(f.getType().equals(int.class))
                                    f.setInt(pf, (int)Double.parseDouble(parameterMap.get(f.getName())[0]));
                                else if(f.getType().equals(boolean.class))
                                    f.setBoolean(pf, Boolean.parseBoolean(parameterMap.get(f.getName())[0]));
                            } catch (IllegalArgumentException ex) {
                                Logger.getLogger(EvolutionSettings.class.getName()).log(Level.SEVERE, null, ex);
                            } catch (IllegalAccessException ex) {
                                Logger.getLogger(EvolutionSettings.class.getName()).log(Level.SEVERE, null, ex);
                            }
                        }
                    }
                    if(pf instanceof AdditionalForm){
                        ((AdditionalForm)pf).handleAdditionalInput(parameterMap);
                    }
                    return;
                }
            }
        }
    }
}