/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package fs;

import ev.Evolution;
import fs.data.DataWrapperAS;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.concurrent.Executors;
import java.util.logging.Level;
import weka.classifiers.Classifier;
import weka.classifiers.bayes.NaiveBayes;
import weka.classifiers.functions.SMO;
import weka.classifiers.lazy.IB1;
import weka.classifiers.lazy.IBk;
import weka.classifiers.trees.J48;

/**
 *
 * @author sigved
 */
public class Benchmark {
    public static double stop = 0.992d;
    public static Map<String,Object> benchmark(EVFS base, Class<? extends Classifier> c, String[] options, int n, int noise) throws Exception{
        DataWrapperAS ds = Test.getRealDataSetWithNominalClass(0, "experimentdata/wine");
//        double best = FeatureSet.allPossibleFeatureSetsSorted(c, ds).last().getFitness();
        double best = stop;
        Set<?>[] sets = new Set<?>[n];
        double[] fit = new double[n];
        double[] counts = new double[n];
        int[] setsize = new int[n];
        double[] stopCV = new double[n];
        double[] time = new double[n];
        ds = Test.getRealDataSetWithNominalClass(0, "experimentdata/wine");
        ds = ds.getBigger(noise, 0);
        if(!testBestIsOk(ds)){
            System.out.println("testBestIsOk failed");
            System.exit(0);
        }
        for(int i = 0; i < n; i++){
            EVFS evfs = new EVFS();
            evfs.seed = i;
            evfs.lp = base.lp;
            evfs.iters = base.iters;
            evfs.stopAt = best-0.0000001d;
            evfs.elitism = base.elitism;
            evfs.mr = base.mr;
            evfs.ps = base.ps;
            evfs.np = base.np;
            evfs.gs = base.gs;
            evfs.rc = base.rc;
            evfs.crosschance = base.crosschance;
            evfs.initpop = base.initpop;
            long t = System.currentTimeMillis();
            evfs.buildClassifier(ds.getAll());
            sets[i] = evfs.indexes;
            setsize[i] = evfs.indexes.size();
            counts[i] = evfs.ff.incr;
            fit[i] = evfs.fit;
            stopCV[i] = evfs.stopCV;
            time[i] = (double)(System.currentTimeMillis()-t);
            double sum = 0;
            for(double ti : time)sum += ti;
            System.out.println(sum/(double)(1+i));
        }
        HashMap<String, Object> ret = new HashMap<String, Object>();
        ret.put("sets",sets);
        ret.put("setsize", setsize);
        ArrayList<Set<Integer>> indlist = new ArrayList<Set<Integer>>();
        for(Set<?> s : sets)indlist.add((Set<Integer>) s);
        double tani = CrossVal.tamimotoDistanceAverage(indlist);
        System.out.println(tani);
        ret.put("tanimoto", tani);
        ret.put("fit",fit);
        ret.put("counts", counts);
        ret.put("stopcv", stopCV);
        ret.put("time", time);
        return ret;
    }
    
    public static void benchmarkElitism() throws Exception{
        EVFS evfs = new EVFS();
        evfs.elitism = 0;
        evfs.ps = 50;
        evfs.lp = 0.0d;
        evfs.iters = 500;
        evfs.mr = 1d;
        evfs.np = 1;
        evfs.rc = 0.0d;
        evfs.gs = 2;
        evfs.crosschance = 0.0d;
        TreeMap<Double, List<Double>> data = new TreeMap<Double, List<Double>>();
        for(int i = 0; i < 10; i++){
            evfs.elitism = i;
            Map<String, Object> r = benchmark(evfs, IBk.class, null, 300,0);
            
            ArrayList<Double> line = new ArrayList<Double>();
            for(double d : (double[])r.get("counts"))line.add(d);
            for(double d : (double[])r.get("fit"))line.add(d);
            
            data.put((double)i, line);
            
            Test.exportCSV(data, "benchmark-elitism");
        }
    }
    
    public static void benchmarkPS() throws Exception{
        EVFS evfs = new EVFS();
        evfs.elitism = 1;
        evfs.lp = 0.0d;
        evfs.iters = 500;
        evfs.mr = 1d;
        evfs.np = 1;
        evfs.rc = 0.0d;
        evfs.gs = 2;
        evfs.crosschance = 0.0d;
        TreeMap<Double, List<Double>> data = new TreeMap<Double, List<Double>>();
        for(int i = 10; i < 250; i+= 10){
            evfs.ps = i;
            evfs.iters = 30000/i;
            Map<String, Object> r = benchmark(evfs, IBk.class, null, 300,0);
            
            ArrayList<Double> line = new ArrayList<Double>();
            for(double d : (double[])r.get("counts"))line.add(d);
            for(double d : (double[])r.get("fit"))line.add(d);
            
            data.put((double)i, line);
            
            Test.exportCSV(data, "benchmark-ps");
        }
    }
    
    public static void benchmarkInitialPop() throws Exception{
        EVFS evfs = new EVFS();
        evfs.elitism = 1;
        evfs.lp = 0.0d;
        evfs.iters = 500;
        evfs.mr = 1d;
        evfs.np = 1;
        evfs.rc = 0.0d;
        evfs.gs = 2;
        evfs.ps = 40;
        evfs.crosschance = 0.0d;
        TreeMap<Double, List<Double>> data = new TreeMap<Double, List<Double>>();
        for(int i = 0; i < 4; i++){
            evfs.initpop = i;
            Map<String, Object> r = benchmark(evfs, IBk.class, null, 300,0);
            
            ArrayList<Double> line = new ArrayList<Double>();
            for(double d : (double[])r.get("counts"))line.add(d);
            for(double d : (double[])r.get("fit"))line.add(d);
            
            data.put((double)i, line);
            
            Test.exportCSV(data, "benchmark-initpop");
        }
    }
    
        
    public static void benchmarkCrossover() throws Exception{
        EVFS evfs = new EVFS();
        evfs.elitism = 1;
        evfs.lp = 0.0d;
        evfs.iters = 500;
        evfs.mr = 1d;
        evfs.np = 2;
        evfs.rc = 0.0d;
        evfs.gs = 2;
        evfs.initpop = 2;
        evfs.ps = 40;
        
        TreeMap<Double, List<Double>> data = new TreeMap<Double, List<Double>>();
        for(double i = 0.0d; i <= 1.0; i+= 0.1d){
            evfs.crosschance = i;
            Map<String, Object> r = benchmark(evfs, IBk.class, null, 300,0);
            
            ArrayList<Double> line = new ArrayList<Double>();
            for(double d : (double[])r.get("counts"))line.add(d);
            for(double d : (double[])r.get("fit"))line.add(d);
            
            data.put((double)i, line);
            
            Test.exportCSV(data, "benchmark-crossover");
        }
    }
    public static void benchmarkNP() throws Exception{
        EVFS evfs = new EVFS();
        evfs.elitism = 1;
        evfs.lp = 0.0d;
        evfs.iters = 500;
        evfs.mr = 1d;
        evfs.np = 2;
        evfs.rc = 0.0d;
        evfs.gs = 2;
        evfs.initpop = 2;
        evfs.ps = 40;
        evfs.crosschance = 1d;
        
        TreeMap<Double, List<Double>> data = new TreeMap<Double, List<Double>>();
        for(int i = 1; i <= 6; i+= 1){
            evfs.np = i;
            Map<String, Object> r = benchmark(evfs, IBk.class, null, 300,0);
            
            ArrayList<Double> line = new ArrayList<Double>();
            for(double d : (double[])r.get("counts"))line.add(d);
            for(double d : (double[])r.get("fit"))line.add(d);
            
            data.put((double)i, line);
            
            Test.exportCSV(data, "benchmark-np");
        }
    }
    public static void benchmarkMutation() throws Exception{
        EVFS evfs = new EVFS();
        evfs.elitism = 1;
        evfs.lp = 0.0d;
        evfs.iters = 250;
        evfs.mr = 1d;
        evfs.np = 2;
        evfs.rc = 0.0d;
        evfs.gs = 2;
        evfs.ps = 40;
        evfs.initpop = 2;
        evfs.crosschance = 0.4;//
        
        TreeMap<Double, List<Double>> data = new TreeMap<Double, List<Double>>();
        for(double i = 0.0d; i <= 1.1; i+= 0.1d){
            evfs.mr = i;
            Map<String, Object> r = benchmark(evfs, IBk.class, null, 300,0);

            ArrayList<Double> line = new ArrayList<Double>();
            for(double d : (double[])r.get("counts"))line.add(d);
            for(double d : (double[])r.get("fit"))line.add(d);
            
            data.put((double)i, line);
            
            Test.exportCSV(data, "benchmark-mutationrate");
        }
    }
    
    public static void benchmarkGS() throws Exception{
        EVFS evfs = new EVFS();
        evfs.elitism = 1;
        evfs.lp = 0.0d;
        evfs.iters = 250;
        evfs.mr = 0.7d;//
        evfs.np = 2;
        evfs.rc = 0.0d;
        evfs.gs = 2;
        evfs.ps = 40;
        evfs.initpop = 2;
        evfs.crosschance = 0.4d;//
        
        TreeMap<Double, List<Double>> data = new TreeMap<Double, List<Double>>();
        for(int i = 1; i <= 10; i+= 1){
            evfs.gs = i;
            Map<String, Object> r = benchmark(evfs, IBk.class, null, 300,0);
            ArrayList<Double> line = new ArrayList<Double>();
            for(double d : (double[])r.get("counts"))line.add(d);
            for(double d : (double[])r.get("fit"))line.add(d);
            
            data.put((double)i, line);
            
            Test.exportCSV(data, "benchmark-gs");
        }
    }
    
    public static void benchmarkRC(int gs) throws Exception{
        EVFS evfs = new EVFS();
        evfs.elitism = 1;
        evfs.lp = 0.0d;
        evfs.iters = 500;
        evfs.mr = 1d;
        evfs.np = 2;
        evfs.rc = 0.0d;
        evfs.gs = gs;
        evfs.initpop = 2;
        evfs.crosschance = 0.4d;//
        evfs.mr = 0.7d;//
        evfs.ps = 40;
        
        TreeMap<Double, List<Double>> data = new TreeMap<Double, List<Double>>();
        for(double i = 0; i <= 1.1; i+= 0.1d){
            evfs.rc = i;
            Map<String, Object> r = benchmark(evfs, IBk.class, null, 300,0);

            ArrayList<Double> line = new ArrayList<Double>();
            for(double d : (double[])r.get("counts"))line.add(d);
            for(double d : (double[])r.get("fit"))line.add(d);
            
            data.put((double)i, line);
            
            Test.exportCSV(data, "benchmark-rc" + gs);
        }
    }
    
    public static void benchmarkNoise(double lp) throws Exception{
        EVFS evfs = new EVFS();
        evfs.elitism = 1;
        evfs.iters = 500;
        evfs.mr = 0.7;//
        evfs.np = 2;
        evfs.rc = 0.2d;//
        evfs.gs = 3;
        evfs.initpop = 2;
        evfs.crosschance = 0.4;//
        evfs.ps = 40;
        evfs.lp = lp;
        
        TreeMap<Double, List<Double>> data = new TreeMap<Double, List<Double>>();
        for(int i = 0; i <= 30; i+= 10){
            Map<String, Object> r = benchmark(evfs, IBk.class, null, 300,i);

            ArrayList<Double> line = new ArrayList<Double>();
            for(double d : (double[])r.get("counts"))line.add(d);
            for(double d : (double[])r.get("stopcv"))line.add(d);
            for(int d : (int[])r.get("setsize"))line.add(d + 0.0);
            line.add( (Double) r.get("tanimoto"));
            
            data.put((double)i, line);
            
            Test.exportCSV(data, "benchmark-noise" + lp);
        }
    }
    
    public static void benchmarkNoiseWithNP(int noise) throws Exception{
        EVFS evfs = new EVFS();
        evfs.elitism = 1;
        evfs.iters = 1000;
        evfs.mr = 0.7;//
        evfs.np = 2;
        evfs.rc = 0.2d;//
        evfs.gs = 3;
        evfs.initpop = 2;
        evfs.crosschance = 0.4;//
        evfs.ps = 40;
        evfs.lp = 0.00000000001d;
        
        TreeMap<Double, List<Double>> data = new TreeMap<Double, List<Double>>();
        for(int i = 2; i <= 10; i++){
            evfs.np = i;
            Map<String, Object> r = benchmark(evfs, IBk.class, null, 300,noise);

            ArrayList<Double> line = new ArrayList<Double>();
            for(double d : (double[])r.get("counts"))line.add(d);
            for(double d : (double[])r.get("stopcv"))line.add(d);
            for(int d : (int[])r.get("setsize"))line.add(d + 0.0);
            line.add( (Double) r.get("tanimoto"));
            
            data.put((double)i, line);
            
            Test.exportCSV(data, "benchmark-noiseWNP" + noise);
        }
    }
    
    public static void benchmarkTable(int noise) throws Exception{
        EVFS evfs = new EVFS();
        evfs.elitism = 1;
        evfs.iters = 500;
        evfs.mr = 0.7;//
        evfs.np = 2;
        evfs.rc = 0.2d;//
        evfs.gs = 3;
        evfs.initpop = 2;
        evfs.crosschance = 0.4;//
        evfs.ps = 40;
        evfs.lp = 0.00000000001;
        
        TreeMap<Double, List<Double>> data = new TreeMap<Double, List<Double>>();
        WrapperFitness.exec = Executors.newFixedThreadPool(1);
        for(int i = 0; i <= 1; i++){
            if(i == 1)WrapperFitness.evalAll = false;
            else WrapperFitness.evalAll = true;
            Map<String, Object> r = benchmark(evfs, IBk.class, null, 300,noise);
            ArrayList<Double> line = new ArrayList<Double>();
            for(double d : (double[])r.get("time"))line.add(d);
            line.add( (Double) r.get("tanimoto"));
            for(double d : (double[])r.get("counts"))line.add(d);
            data.put((double)i, line);
            
            Test.exportCSV(data, "benchmark-table" + noise);
        }
    }
    
    public static void benchmarkMT(int ps) throws Exception{
        EVFS evfs = new EVFS();
        evfs.elitism = 1;
        evfs.iters = 500;
        evfs.mr = 0.7;//
        evfs.np = 2;
        evfs.rc = 0.2d;
        evfs.gs = 3;
        evfs.initpop = 2;
        evfs.crosschance = 0.4;//
        evfs.elitism = 1;//changed
        evfs.ps = ps;
        evfs.lp = 0.00000000001;
        if(ps > 40){
//            evfs.rc = 0.2d;
            evfs.gs = 4;
//            evfs.elitism = 6;
        }
        
        TreeMap<Double, List<Double>> data = new TreeMap<Double, List<Double>>();
        for(int i = 1; i < 13; i++){
            WrapperFitness.exec = Executors.newFixedThreadPool(i);
            Map<String, Object> r = benchmark(evfs, IBk.class, null, 300,0);

            ArrayList<Double> line = new ArrayList<Double>();
            for(double d : (double[])r.get("time"))line.add(d);
            
            for(double d : (double[])r.get("stopcv"))line.add(d);
            data.put((double)i, line);
            
            Test.exportCSV(data, "benchmark-mt"+ps);
        }
    }
    
    public static void benchmarkBig() throws Exception{
        EVFS evfs = new EVFS();
        evfs.elitism = 4;//changed
        evfs.iters = 250;//changed
        evfs.mr = 1d;
        evfs.np = 2;
        evfs.rc = 0.2d;
        evfs.gs = 4;//changed
        evfs.initpop = 3;//changed
        evfs.crosschance = 1d;
        evfs.ps = 80;
        evfs.lp = 0.000000000001d;
        evfs.servlet = true;
        evfs.classifier = new IB1();
        
        TreeMap<Double, List<Double>> data = new TreeMap<Double, List<Double>>();
        DataWrapperAS ds = Test.getRealDataSetWithNominalClass(0, 12601,' ',"experimentdata/lc");
//        DataWrapperAS ds = Test.getRealDataSetWithNominalClass(6, "experimentdata/bupa");
        evfs.buildClassifier(ds.getAll());
        
        
    }
    
    
    static FeatureSet fs = new FeatureSet(1, 20, null);
    static {
        fs.classifier = IBk.class;
        fs.indexes = new HashSet<Integer>();
        fs.indexes.add(0);
        fs.indexes.add(1);
        fs.indexes.add(4);
        fs.indexes.add(6);
        fs.indexes.add(7);
        fs.indexes.add(9);
        fs.indexes.add(10);
        fs.indexes.add(12);
    }
    
    /**
     * test that checks the optimal solution still works
     * @param dw
     * @return 
     */
    public static boolean testBestIsOk(DataWrapperAS dw){
        WrapperFitness wf = new WrapperFitness(dw);
        
        
        
        HashSet<FeatureSet> fss = new HashSet<FeatureSet>();
        fss.add(fs);
        wf.runOnPopulation(fss);
        
        return fs.getFitness() > 0.994d;
    }
    
    public static void main(String[] args) throws Exception{
        WrapperFitness.evalAll = true;
        Evolution.log.setLevel(Level.SEVERE);
//        EVFS evfs = new EVFS();
//        evfs.ps = 50;
//        evfs.lp = 0.0d;
//        evfs.iters = 500;
//        evfs.mr = 1d;
//        evfs.np = 1;
//        evfs.rc = 0.0d;
//        evfs.gs = 2;
//        benchmark(evfs, IBk.class, null, 10);
//        benchmarkInitialPop();
//        benchmarkNP();
//        WrapperFitness.evalAll = true;
//        benchmarkElitism();
//        benchmarkPS();
//        benchmarkInitialPop();
//        benchmarkCrossover();
//        benchmarkNP();
//        benchmarkMutation();
//        benchmarkGS();
//        benchmarkRC(2);
//        benchmarkRC(3);
//        benchmarkNoise(0.0d);
//        benchmarkNoise(0.00000000001);
//        benchmarkNoise(0.001);
//        benchmarkNoise(0.002);
//        stop = 0.988888d;
//        benchmarkNoise(0.003);
//        benchmarkNoiseWithNP(0);
//        benchmarkNoiseWithNP(30);
//        benchmarkNoiseWithNP(60);
        stop = 0.994d;
//        benchmarkTable(0);
//        WrapperFitness.evalAll = false;
//        benchmarkMT(40);
//        infoGraphics();
//        benchmarkMT(80);
//        benchmarkBig();
        System.out.print("DONE!");
    }

    private static void infoGraphics() throws Exception {
        EVFS evfs = new EVFS();
        evfs.servlet = true;
        evfs.initpop = 3;
        DataWrapperAS ds = Test.getRealDataSetWithNominalClass(0, 12601,' ',"experimentdata/lc");
        evfs.buildClassifier(ds.getAll());
    }
}