
package fs;

import ev.Evolution;
import ev.cc.ChildCreator;
import ev.cc.parentSelectors.TournamentSelector;
import ev.popfunc.AgeAdder;
import ev.popfunc.PopulationLimit;
import ev.ui.EvolutionPanel;
import ev.ui.servlet.EVServer;
import fs.data.DataWrapper.TYPE;
import fs.data.DataWrapperAS;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFrame;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import weka.classifiers.Classifier;
import weka.classifiers.bayes.NaiveBayes;
import weka.classifiers.functions.SMO;
import weka.classifiers.lazy.IBk;
import weka.classifiers.trees.J48;

/**
 *
 * @author sigved
 */
public class Main {
    static boolean windowMode = false;
    static String dataFile = "experimentdata/wine";
    static int noise = 400;
    static int ps = 400;
    static double mr = 0.5d;
    static int age = 0;
    static int elitism = 1;
    static double lp = 0.000000000001d;
    static String port = "4242";
//    private static int classIndex = 8;
    
    
    private static void handleArgs(String[] args) {
        try{
            for(int i = 0; i < args.length; i++){
                if(args[i].equals("-w")){
                    windowMode = true;
                }else if(args[i].equals("-f")){
                    i++;
                    dataFile = args[i];
                }else if(args[i].equals("-noise")){
                    i++;
                    noise = Integer.parseInt(args[i]);
                }else if(args[i].equals("-ps")){
                    i++;
                    ps = Integer.parseInt(args[i]);
                }else if(args[i].equals("-mr")){
                    i++;
                    mr = Double.parseDouble(args[i]);
                }else if(args[i].equals("-age")){
                    i++;
                    age = Integer.parseInt(args[i]);
                }else if(args[i].equals("-elitism")){
                    i++;
                    elitism = Integer.parseInt(args[i]);
                }else if(args[i].equals("-lp")){
                    i++;
                    lp = Double.parseDouble(args[i]);
                }else if(args[i].equals("-port")){
                    i++;
                    port = args[i];
                }else{
                    printHelp();
                    System.exit(0);
                }
            }
        }catch(Exception e){
            System.out.println("invalid arguments");
            printHelp();
            System.exit(1);
        }
    }
    
    
    private static void printHelp() {
        System.out.println("\n-w for window mode");
        System.out.println("-f filename, to select input data");
        System.out.println("-noise integer, to define amount of noise variables");
        System.out.println("-ps integer, defines popluation size");
        System.out.println("-mr float, define mutation rate");
        System.out.println("-age integer, defines population age limit, won't affect elitism phenos");
        System.out.println("-elitism integer, defines elite group size");
        System.out.println("-lp float, defines length penalty");
        System.out.println("\nundefined variables use default variables");
    }
    
    public static void main(String[] args) throws FileNotFoundException, IOException, Exception{
        handleArgs(args);
        
        DataWrapperAS dw = Test.getRealDataSetWithNominalClass(0, dataFile);
        dw = dw.getBigger(noise, 0);
        
        WrapperFitness ff = new WrapperFitness(dw);
        ff.nf = 10;
        
        ArrayList<Class<? extends Classifier>> classifiers = new ArrayList<Class<? extends Classifier>>();
        classifiers.add(IBk.class);
        System.out.println(classifiers);
        Evolution<FeatureSet,FeatureSet> e = FeatureSet.createFeaturePopulation(0,ps, classifiers, dw.numIndexes()-1, true);
        
        e.getFunctions().add(new AgeAdder());
        ChildCreator cc = new ChildCreator();
        cc.mutationRate = mr;
        cc.numberOfChildren = ps;
        cc.numberOfParents = 2;
        TournamentSelector ts = new TournamentSelector();
        ts.groupSize = 3;
        ts.randomChance = 0.325d;
        cc.setSelector(ts);
        e.getFunctions().add(cc);
        ff.lp = lp;
        ff.nf = 10;

        PopulationLimit pl = new PopulationLimit();
        pl.limit = elitism;
        pl.maxAge = 1000000000;
        pl.minAge = age;
        e.getFunctions().add(pl);
        e.getFunctions().add(ff);
        e.getFunctions().add(pl);
        
        if(windowMode){
            EvolutionPanel ep = new EvolutionPanel();
            JFrame f = new JFrame();
            f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
            f.add(ep);
            f.pack();
            f.setVisible(true);
            ep.setModel(e);
        }else{
            try {
                EVServer server = new FSServer(Integer.parseInt(port), e, dw);
                server.run();
            } catch (Exception ex) {
                Logger.getLogger(FeatureSet.class.getName()).log(Level.SEVERE, null, ex);
            }
            System.exit(0);
        }
    }

    private static void checkLibsvm() {
        Classifier lsvm = new SMO();
        for(String s : lsvm.getOptions())
            System.out.println(s);
    }
}
