
package fs;

import ev.Evolution;
import ev.cc.ChildCreator;
import ev.cc.parentSelectors.TournamentSelector;
import ev.popfunc.AgeAdder;
import ev.popfunc.PopulationLimit;
import fs.data.DataSourceAS;
import fs.data.DataWrapper.TYPE;
import fs.data.DataWrapperAS;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.logging.Level;
import java.util.logging.Logger;
import weka.classifiers.Classifier;
import weka.classifiers.Evaluation;
import weka.classifiers.bayes.NaiveBayes;
import weka.classifiers.functions.MultilayerPerceptron;
import weka.classifiers.functions.SMO;
import weka.classifiers.lazy.IBk;
import weka.classifiers.trees.J48;
import weka.core.Instances;

/**
 *
 * @author sigved
 */
public class Test {
    static ArrayList<String> statsOut = new ArrayList<String>();

    
    public static void testForPaper2(int iters, int ps, int elitism, double lp, double mr, int noise) throws Exception{
        System.out.println("test2");
        //TEST SMO
        ArrayList<Class<? extends Classifier>> cllist = new ArrayList<Class<? extends Classifier>>();
        cllist.add(IBk.class);
        cllist.add(EVFS.class);
//        cllist.add(&MultilayerPerceptron.class);
        cllist.add(SMO.class);
        cllist.add(J48.class);
        cllist.add(NaiveBayes.class);
        DataWrapperAS orig;
//        orig = getRealDataSetWithNominalClass(11, "winequality-white.csv");
//        testOnDataSet(orig,smolist, iters, ps, elitism, lp, mr, noise);
        ArrayList<Class<? extends Classifier>> cl = new ArrayList<Class<? extends Classifier>>();
        
        statsOut.add("\n\niters(" + iters + ") ps(" + ps + ") elitism(" + elitism + ") lp(" + lp + ") mr(" + mr + ") noise(" + noise + ")");
        for(Class<? extends Classifier> c : cllist){
            cl.clear();
            cl.add(c);
            statsOut.add(""+c+":");
            
            statsOut.add("wdbc");
            orig = getRealDataSetWithNominalClass(1, "experimentdata/wdbc");
            testOnDataSet(orig,cl, iters, ps, elitism, lp, mr, noise);
            
            orig = getRealDataSetWithNominalClass(0, "experimentdata/lunggenes");
            testOnDataSet(orig,cl, iters, ps, elitism, lp, mr, noise);
            
//            statsOut.add("ionosphere");
//            orig = getRealDataSetWithNominalClass(34, "experimentdata/ionosphere");
//            testOnDataSet(orig,cl, iters, ps, elitism, lp, mr, noise);
//
//            statsOut.add("bupa");
//            orig = getRealDataSetWithNominalClass(6, "experimentdata/bupa");
//            testOnDataSet(orig,cl, iters, ps, elitism, lp, mr, noise);
//
//            statsOut.add("pima");
//            orig = getRealDataSetWithNominalClass(8, "experimentdata/pima");
//            testOnDataSet(orig,cl, iters, ps, elitism, lp, mr, noise);
//            
//            statsOut.add("wine");
//            orig = getRealDataSetWithNominalClass(0, "experimentdata/wine");
//            testOnDataSet(orig,cl, iters, ps, elitism, lp, mr, noise);
//            
//            statsOut.add("iris");
//            orig = getRealDataSetWithNominalClass(4, "experimentdata/iris");
//            testOnDataSet(orig,cl, iters, ps, elitism, lp, mr, noise);
//            
//            statsOut.add("sonar");
//            orig = getRealDataSetWithNominalClass(60, "experimentdata/sonar");
//            testOnDataSet(orig,cl, iters, ps, elitism, lp, mr, noise);
//            
//            statsOut.add("vote");
//            orig = getRealDataSetWithNominalClass(0, "experimentdata/vote");
//            testOnDataSet(orig,cl, iters, ps, elitism, lp, mr, noise);
            
        }
    }
    
    public static DataWrapperAS getRealDataSetWithNominalClass(int classIndex, String datafile) throws IOException, Exception{
        DataWrapperAS odw = new DataWrapperAS(datafile);
        TYPE[] tarr = odw.getTYPEArray();
        tarr[classIndex] = TYPE.NOMINAL;
        DataWrapperAS ret = new DataWrapperAS(datafile, tarr);
        
        ret.setClassIndex(classIndex);
        System.out.println(ret.getFitnessAndTraining().numInstances());
        return ret;
    }
    
    static DataWrapperAS getRealDataSetWithNominalClass(int classIndex, int n, char c, String datafile) throws IOException, Exception {
        TYPE[] tarr = new TYPE[n];
        tarr[classIndex] = TYPE.NOMINAL;
        for(int i = 0; i < tarr.length; i++)
            if(i != classIndex)
                tarr[i] = TYPE.REAL;
        
        DataWrapperAS ret = new DataWrapperAS(datafile, tarr, new char[]{c});
        
        ret.setClassIndex(classIndex);
        System.out.println(ret.getFitnessAndTraining().numInstances());
        return ret;
    }
    
    /**
     * @param data: original data
     * @param classifiers: classifiers to use
     * @param ps: population size
     * @param elitism: how many of the best phenos will survive
     * @param lp: length penalty for each attribute
     * @param mr: mutation rate
     * @param noise: noise variables
     */
    public static void testOnDataSet(DataWrapperAS data, ArrayList<Class<? extends Classifier>> classifiers, int iterations, int ps, int elitism, double lp, double mr, int noise){
        DataWrapperAS big = data.getBigger(noise, 0);
        testSimpleClassifiers(classifiers, data, noise);
        testSimpleClassifiers(classifiers, big, noise);
//        testEVClassifier(big, classifiers, iterations, ps, elitism, lp, mr, noise);
//        testEVClassifier(data, classifiers, iterations, ps, elitism, lp, mr, noise);
    }
    
    private static void testSimpleClassifiers(ArrayList<Class<? extends Classifier>> classifiers, DataWrapperAS dw, int noise){
        for(Class<? extends Classifier> c : classifiers){
            try {
                Classifier classifier = c.newInstance();
                Instances is = dw.getFitnessAndTraining();
                classifier.buildClassifier(is);
                Evaluation eval = new Evaluation(is);
                eval.evaluateModel(classifier, dw.getValidation());
                statsOut.add(classifier.getClass() + ", " + dw.getTraining().numAttributes() + " attributes: " + eval.errorRate());
            } catch (Exception ex) {
                statsOut.add("failed to evaluate simple classifier of type " + c);
                Logger.getLogger(Test.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    private static void testEVClassifier(DataWrapperAS big, ArrayList<Class<? extends Classifier>> classifiers, int iterations, int ps, int elitism, double lp, double mr, int noise) {
        Evolution<FeatureSet,FeatureSet> e = FeatureSet.createFeaturePopulation(0,ps, classifiers, big.numIndexes()-1, true);
        WrapperFitness ff = new WrapperFitness(big);
        e.getFunctions().add(new AgeAdder());
        ChildCreator cc = new ChildCreator();
        cc.mutationRate = mr;
        cc.numberOfChildren = ps;
        cc.numberOfParents = 2;
        TournamentSelector ts = new TournamentSelector();
        ts.groupSize = 20;
        cc.setSelector(ts);
        e.getFunctions().add(cc);
        ff.lp = lp;
        
        PopulationLimit pl = new PopulationLimit();
        pl.limit = elitism;
        pl.maxAge = 1000000000;
        pl.minAge = 0;
        
        e.getFunctions().add(pl);
        e.getFunctions().add(ff);
        e.getFunctions().add(pl);
        
        e.evolve(iterations);
        
        FeatureSet fs = (FeatureSet) e.getBestPheno();
        try {
            double err = testFeatureSet(fs, big);
            statsOut.add("EV(" + classifiers + ", "+ fs.indexes + "), " + big.getTraining().numAttributes() + " attributes: " + err);
        } catch (Exception ex) {
            statsOut.add("EV(" + classifiers + "), failed to evaluate");
        }
    }
    
    public static void main(String[] args) throws Exception{
//        testForPaper(100, 400, 1, 0.0001f, 0.1d, 400);
//        testForPaper2(50, 50, 1, 0.00000000d, 0.1d, 400);
//        for(String s : statsOut)System.out.println(s);
//        System.exit(0);
//          DataSourceAS ds = getRealDataSetWithNominalClass(0, "experimentdata/lunggenes");
//          TreeSet<FeatureSet> fss = FeatureSet.allPossibleFeatureSetsSorted(IBk.class, ds);
//          for(FeatureSet fs: fss){
//              System.out.println(fs.getFitness() + "->" +fs.indexes);
//          }
        
//        for(Set<Integer> ints : getAllPossibleSubsets(4)){
//            System.out.println(ints);
//        }
        
    }

    public static double testFeatureSet(FeatureSet fs, DataWrapperAS big) throws Exception {
            Classifier c = fs.classifier.newInstance();
            HashSet<Integer> allIndexes = new HashSet<Integer>();
            int ci = big.getClassIndex();
            for(Integer i : fs.indexes)if(i < ci)allIndexes.add(i);else allIndexes.add(i+1);
            allIndexes.add(ci);
            DataSourceAS nds = big.getFeatureSubSet(allIndexes);
            Instances trainInstances = nds.getFitnessAndTraining();
//            Instances trainInstances = nds.getFitness();
            c.buildClassifier(trainInstances);
            Evaluation eval = new Evaluation(trainInstances);
            eval.evaluateModel(c, nds.getValidation());
            return eval.errorRate();
    }
    
    /**
     * create all possible subset, including the improper one
     * @param indexes
     * @param current
     * @return 
     */
    public static Set<Set<Integer>>getAllPossibleSubsets(Set<Integer> indexes, Set<Integer> current){
        HashSet<Set<Integer>> ret = new HashSet<Set<Integer>>();
        ret.add(current);
        HashSet<Integer> cc = new HashSet<Integer>(indexes);
        for(int i : indexes){
            Set<Integer> ns = new HashSet<Integer>(current);
            ns.add(i);
            cc.remove(i);
            Set<Integer> ni = new HashSet<Integer>(cc);
            ret.addAll(getAllPossibleSubsets(ni, ns));
        }
        return ret;
    }
    
    public static void exportCSV(Map m, String fn) throws FileNotFoundException{
        PrintWriter out = new PrintWriter(fn);
        ArrayList<Map.Entry> e = new ArrayList(m.entrySet());
        for(int i = 0; i < e.size(); i++)out.println(e.get(i).getKey() + ", " + e.get(i).getValue());
        out.close();
    }

    public static Set<Set<Integer>> getAllPossibleSubsets(int n) {
        HashSet<Integer> indexes = new HashSet<Integer>();
        for(int i = 0; i < n; i++)indexes.add(i);
        return getAllPossibleSubsets(indexes, new HashSet<Integer>());
    }


}