/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package fs;

import ev.Evolution;
import fs.data.DataWrapperAS;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import weka.classifiers.Classifier;
import weka.classifiers.bayes.NaiveBayes;
import weka.classifiers.functions.MultilayerPerceptron;
import weka.classifiers.functions.SMO;
import weka.classifiers.lazy.IB1;
import weka.classifiers.lazy.IBk;
import weka.classifiers.trees.J48;
import weka.core.Instances;

/**
 *
 * @author sigved
 */
public class ThesisTests {
    final static DecimalFormat nf = new DecimalFormat(".###");
    
    static{
        Evolution.log.setLevel(Level.SEVERE);
    }
    
    final static ArrayList<Class<? extends Classifier>> wrappers = new ArrayList<Class<? extends Classifier>>();
    static{
        wrappers.add(NaiveBayes.class);
//        wrappers.add(IB1.class);
        wrappers.add(IBk.class);
//        wrappers.add(J48.class);
//        wrappers.add(SMO.class);
//        wrappers.add(MultilayerPerceptron.class);
    }
    
    final static HashMap<String, Integer> uciData = new HashMap<String, Integer>();
    static{
        uciData.put("experimentdata/bupa", 6);
        uciData.put("experimentdata/ionosphere", 34);
        uciData.put("experimentdata/iris", 4);
        uciData.put("experimentdata/pima", 8);
        uciData.put("experimentdata/sonar", 60);
        uciData.put("experimentdata/vote", 0);
        uciData.put("experimentdata/wine", 0);
        uciData.put("experimentdata/wdbc", 1);
    }
    
    public static CrossVal testOn(DataWrapperAS dw, String[] options) throws Exception{
        CrossVal cv = CrossVal.crossval(EVFS.class, options, 10, dw.getAll());
        return cv;
    }
    
    public static void main(String[] inn) throws Exception{
//        uciTests();
//        noisyUciTests();
//        GemSysTests();
        trainTest();
    }

    private static void uciTests() throws Exception {
        for(Class<? extends Classifier> w : wrappers){
            String[] options = new String[]{"-w", w.getCanonicalName()};
            String[] top = new String[]{"data-set", "\\# features", "original", "error", "fitness", "\\# selected", "tanimoto"};
            List<String[]> data = new ArrayList<String[]>();
            for(Map.Entry<String, Integer> e : uciData.entrySet()){
                DataWrapperAS dw = Test.getRealDataSetWithNominalClass(e.getValue(), e.getKey());
                CrossVal cv = testOn(dw, options);
                String es = nf.format(cv.mean) + "(" + nf.format(cv.std) + ")";
                String fs = nf.format(cv.indexesMean) + "(" + nf.format(cv.indexesStd) + ")";
                String fit = nf.format(1d-cv.fitnessMean) + "(" + nf.format(cv.fitnessStd) + ")";
                String tani = nf.format(cv.tamimotoDistanceAverage());
                CrossVal ocv = CrossVal.crossval(w, null, 10, dw.getAll());
                String orig = nf.format(ocv.mean) + "(" + nf.format(ocv.std) + ")";
                data.add(new String[]{e.getKey().substring(15), ""+dw.numIndexes(), orig, es, fit, fs, tani});
            }
            texExport(top, data, "uciTests-" + w + ".tex");
        }
    }
    final static int noiseLevel = 400;
    private static void noisyUciTests() throws Exception{
        for(Class<? extends Classifier> w : wrappers){
            String[] options = new String[]{"-w", w.getCanonicalName()};
            String[] top = new String[]{"data-set", "\\# features", "original", "error", "fitness", "\\# selected", "\\# noise", "tanimoto"};
            List<String[]> data = new ArrayList<String[]>();
            for(Map.Entry<String, Integer> e : uciData.entrySet()){
                DataWrapperAS dw = Test.getRealDataSetWithNominalClass(e.getValue(), e.getKey()).getBigger(noiseLevel, 0);
                CrossVal cv = testOn(dw, options);
                String es = nf.format(cv.mean) + "(" + nf.format(cv.std) + ")";
                String fs = nf.format(cv.indexesMean) + "(" + nf.format(cv.indexesStd) + ")";
                String fit = nf.format(1d-cv.fitnessMean) + "(" + nf.format(cv.fitnessStd) + ")";
                
                //noise stat
                double noise = 0;
                double[] localNoises = new double[10];
                int c = 0;
                for(Set<Integer> is : cv.indexSet){
                    double localNoise = 0;
                    for(int i : is)if(i > dw.numIndexes() - noiseLevel)localNoise++;
                    noise += localNoise/10d;
                    localNoises[c] = localNoise;
                    c++;
                }
                
                double t = 0d;
                for(double ln : localNoises)t += (ln-noise)*(ln-noise);
                double noiseStd = Math.sqrt(t/10d);
                String ns = nf.format(noise) + "(" + nf.format(noiseStd) + ")";
                
                
                String tani = nf.format(cv.tamimotoDistanceAverage());
                CrossVal ocv = CrossVal.crossval(w, null, 10, dw.getAll());
                String orig = nf.format(ocv.mean) + "(" + nf.format(ocv.std) + ")";
                data.add(new String[]{e.getKey().substring(15), ""+dw.numIndexes(), orig, es, fit, fs, ns, tani});
            }
            texExport(top, data, "noisyUciTests-" + w + ".tex");
        }
    }

    private static void GemSysTests() throws Exception{
        List<String> fnl = new ArrayList<String>();
        List<Character> sl = new ArrayList<Character>();
        List<Integer> fsl = new ArrayList<Integer>();
        List<Integer> cil = new ArrayList<Integer>();
        
//        fnl.add("experimentdata/lc");sl.add(' ');fsl.add(12601);cil.add(0);
//        fnl.add("experimentdata/pt");sl.add(' ');fsl.add(10510);cil.add(0);
//        fnl.add("experimentdata/l1");sl.add(' ');fsl.add(5328);cil.add(0);
//        fnl.add("experimentdata/l2");sl.add(' ');fsl.add(11226);cil.add(0);
//        fnl.add("experimentdata/bt1");sl.add(' ');fsl.add(5921);cil.add(0);
//        fnl.add("experimentdata/bt2");sl.add(' ');fsl.add(10368);cil.add(0);
//        fnl.add("experimentdata/t9");sl.add(' ');fsl.add(5727);cil.add(0);
//        fnl.add("experimentdata/t11");sl.add(' ');fsl.add(12534);cil.add(0);
//        fnl.add("experimentdata/t14");sl.add(' ');fsl.add(15010);cil.add(0);
//        fnl.add("experimentdata/dl");sl.add(' ');fsl.add(5470);cil.add(0);
//        fnl.add("experimentdata/sr");sl.add(' ');fsl.add(2309);cil.add(0);
//        fnl.add("experimentdata/leukemia");sl.add(',');fsl.add(7130);cil.add(7129);
//        fnl.add("experimentdata/col.new");sl.add(' ');fsl.add(2001);cil.add(0);
        fnl.add("experimentdata/lymph.new");sl.add(' ');fsl.add(4027);cil.add(0);
        
        for(Class<? extends Classifier> w : wrappers){
            String[] options = new String[]{"-w", w.getCanonicalName()};
            String[] top = new String[]{"data-set", "\\# features", "error", "fitness", "\\# selected", "tanimoto"};
            List<String[]> data = new ArrayList<String[]>();
            for(int i = 0; i < fnl.size(); i++){
                DataWrapperAS dw = Test.getRealDataSetWithNominalClass(cil.get(i), fsl.get(i),sl.get(i),fnl.get(i));
                CrossVal cv = testOn(dw, options);
                String es = nf.format(cv.mean) + "(" + nf.format(cv.std) + ")";
                String fs = nf.format(cv.indexesMean) + "(" + nf.format(cv.indexesStd) + ")";
                String fit = nf.format(1d-cv.fitnessMean) + "(" + nf.format(cv.fitnessStd) + ")";
                String tani = nf.format(cv.tamimotoDistanceAverage());
                CrossVal ocv = CrossVal.crossval(w, null, 10, dw.getAll());
//                String orig = nf.format(ocv.mean) + "(" + nf.format(ocv.std) + ")";
                data.add(new String[]{fnl.get(i).substring(15), ""+dw.numIndexes(), es, fit, fs, tani});
            }
            texExport(top, data, "bigTests-" + w + ".tex");
        }
    }
    
    public static void trainTest() throws Exception{
                List<String> fnl = new ArrayList<String>();
        List<Character> sl = new ArrayList<Character>();
        List<Integer> fsl = new ArrayList<Integer>();
        List<Integer> cil = new ArrayList<Integer>();
        
        fnl.add("experimentdata/leukemia");sl.add(',');fsl.add(7130);cil.add(7129);
        fnl.add("experimentdata/col.new");sl.add(' ');fsl.add(2001);cil.add(0);
        fnl.add("experimentdata/lymph.new");sl.add(' ');fsl.add(4027);cil.add(0);
        ArrayList<Integer> splitPoints = new ArrayList<Integer>();
        splitPoints.add(38);
        splitPoints.add(31);
        splitPoints.add(48);
        
        
        for(Class<? extends Classifier> w : wrappers){
            String[] options = new String[]{"-w", w.getCanonicalName()};
            String[] top = new String[]{"data-set", "\\# features", "error", "fitness", "\\# selected", "tanimoto"};
            List<String[]> data = new ArrayList<String[]>();
            for(int i = 0; i < fnl.size(); i++){
                DataWrapperAS dw = Test.getRealDataSetWithNominalClass(cil.get(i), fsl.get(i),sl.get(i),fnl.get(i));
//                CrossVal cv = testOn(dw, options);
//                CrossVal ocv = CrossVal.crossval(w, null, 10, dw.getAll());
                double sumE = 0d;
                double sumF = 0d;
                double sumI = 0d;
                double[] errarr = new double[10];
                for(int k = 0; k < 10; k++){
                    EVFS evfs = new EVFS();
                    evfs.classifier = w.newInstance();
                    evfs.seed = k;
                    Instances train = new Instances(dw.getAll(), 0, splitPoints.get(i));
                    evfs.buildClassifier(train);
                    Instances test = new Instances(dw.getAll(), splitPoints.get(i), dw.getAll().numInstances()-splitPoints.get(i));
                    double sumFails = 0;
                    for(int j = 0; j < test.numInstances(); j++){
                        if(evfs.classifyInstance(test.instance(j)) != test.instance(j).classValue()){
                            sumFails++;
                        }
                    }
                    sumI += evfs.indexes.size()-1;
                    sumF += evfs.fit;
                    sumE += (sumFails/(double)test.numInstances());
                    errarr[k] = (sumFails/(double)test.numInstances());
                }
                System.out.println();
                for(int k = 0; k < 10; k++)System.out.print(errarr[k] + " ");
                System.out.println();
                String es = nf.format(sumE/10d);
                String fs = nf.format(sumI/10d);
                String fit = nf.format(sumF/10d);
//                String orig = nf.format(ocv.mean) + "(" + nf.format(ocv.std) + ")";
                data.add(new String[]{fnl.get(i).substring(15), ""+dw.numIndexes(), es, fit, fs});
            }
            texExport(top, data, "testvalidate-" + w + ".tex");
        }
    }
    
    public static void texExport(String[] top, List<String[]> data, String fn) throws FileNotFoundException{
        PrintWriter out = new PrintWriter(fn);
        
        out.println("\\begin{table}[h!]");
        out.println("\\begin{center}");
        out.println("\\caption{" + fn + "}");
        out.println("\\label{" + fn + "}");
        out.print("\\begin{tabular}{|");
        for(int i = 0; i < top.length; i++)out.print("c|");
        out.println("}\n\\hline");
        for(int i = 0; i < top.length; i++){
            out.print(top[i]);
            if(i < top.length - 1)out.print("&");
            else out.println("\\\\");
        }
        out.println("\\hline");
        for(String[] l : data){
            for(int i = 0; i < l.length; i++){
                out.print(l[i]);
                if(i < l.length - 1)out.print("&");
                else out.println("\\\\");
            }
        }
        out.println("\\hline");
        out.println("\\end{tabular}");
        out.println("\\end{center}");
        out.println("\\end{table}");
        
        out.close();
    }
}