/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package fs;

import ev.PopulationFunction;
import fs.data.DataSourceAS;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author sigved
 */
public class WrapperFitness extends PopulationFunction<FeatureSet>{
    static public ExecutorService exec = Executors.newFixedThreadPool(Runtime.getRuntime().availableProcessors());//thread pool for multithreading
    DataSourceAS dw;//the source of the trainging data
    int nf = 10;//number of folds for the fitness evaluation
    double lp = 0d;//how much to subtract for each feature from the fitness
    Integer incr = 0;//how many ecaluations this instance has performed, for debugging
    
    public WrapperFitness(DataSourceAS dw){
        this.dw = dw;
    }
    @Override
    public Set<FeatureSet> runOnPopulation(Set<FeatureSet> pop) {
        ArrayList<Callable<Object>> r = new ArrayList<Callable<Object>>();
        for(final FeatureSet fs : pop){
            if(fs.getFitness() <= 0){
                r.add(new Callable<Object>() {
                    @Override
                    public Object call() throws Exception {
                        long start = System.currentTimeMillis();
                        fs.setFitness(fitness(fs));
                        return null;
                    }
                });
            }
        }
        try{
            exec.invokeAll(r);
        }catch (InterruptedException ex) {
            Logger.getLogger(WrapperFitness.class.getName()).log(Level.SEVERE, null, ex);
        }
        return pop;
    }
    
    public final HashMap<Set<Integer>, Double> fitnessMap = new HashMap<Set<Integer>, Double>();//performance trick
    public static boolean evalAll = false;
    
    /**
     * Calculate the fitness of one FeatureSet.
     * @param fs: proposed feature subset solution
     * @return the fitness value
     */
    private double fitness(FeatureSet fs) {
        try {
            if(!fitnessMap.containsKey(fs.indexes) || evalAll){
                HashSet<Integer> indexes = new HashSet<Integer>();
                int ci = dw.getClassIndex();

                for(Integer i : fs.indexes)
                    if(i < ci)indexes.add(i);
                    else indexes.add(i+1);
                indexes.add(ci);

                DataSourceAS nds = dw.getFeatureSubSet(indexes);
                CrossVal cv = CrossVal.crossval(fs.classifier, new String[0], nf, nds.getAll());

                
                synchronized(fitnessMap){
                    incr++;
                    fitnessMap.put(fs.indexes, 1.0d - cv.mean - (lp*fs.indexes.size()));// - cv.indexesMean;
                }
            }else{
//                System.out.println("skip evaluation :D");
            }
            synchronized(fitnessMap){
                return fitnessMap.get(fs.indexes);
            }
        } catch (Exception ex) {
            return 0d;
        }
    }
    
    @Override
    public String toString(){
        return "WrapperFitness(" + this.lp + ", " + this.nf + ") ";
    }
}
