/*
 * I OWN ALL THE SYNTAX
 */
package fs.data;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Random;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import sun.reflect.generics.reflectiveObjects.NotImplementedException;
import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;

/**
 *
 * @author sigved
 */
public class DataWrapper{
    private static char[] defSep = new char[]{',',';','|'};

    public int numIndexes() {
        return training.numAttributes();
    }
    public static enum TYPE{NOMINAL, REAL};
    
    private FastVector dataVector;
    Instances training;
    Instances fitness;
    Instances validation;
    
    public synchronized Instances getTrainingInstances(Set<Integer> indexes) {
        int[] is = new int[indexes.size()+1];
        int k = 0;
        for(Integer i : indexes){
            is[k] = i;k++;if(i >= training.classIndex())is[k-1]++;
//            System.out.print(i + "/" + is[k-1] + "|");
        }
        is[k] = training.classIndex();
//        System.out.println(is[k]);
        return featureSubSet(training, is);
    }

    public synchronized Instances getValidationInstances(Set<Integer> indexes) {
        int[] is = new int[indexes.size()+1];
        int k = 0;
        for(Integer i : indexes){
            is[k] = i;k++;if(i >= training.classIndex())is[k-1]++;
        }
        is[k] = training.classIndex();
        return featureSubSet(validation, is);
    }
    
    public synchronized Instances getFitnessInstances(Set<Integer> indexes) {
        int[] is = new int[indexes.size()+1];
        int k = 0;
        for(Integer i : indexes){
            is[k] = i;k++;if(i >= training.classIndex())is[k-1]++;
        }
        is[k] = training.classIndex();
        return featureSubSet(fitness, is);
    }
    public DataWrapper(String fn, TYPE[] typeVector, char[] separators) throws FileNotFoundException, IOException{
        init(fn, typeVector, separators);
    }
    public DataWrapper(String fn, TYPE[] typeVector) throws FileNotFoundException, IOException{
        init(fn, typeVector, defSep);
    }
    public DataWrapper(String fn, char[] separators) throws FileNotFoundException, IOException{
        init(fn, separators);
    }
    public DataWrapper(String fn) throws FileNotFoundException, IOException{
        init(fn, defSep);
    }
    private DataWrapper(){
        
    }
    
    /**
     * best effort to construct a type vector, when it's missing
     * @param fn
     * @param separators
     * @throws FileNotFoundException
     * @throws IOException 
     */
    private void init(String fn, char[] separators) throws FileNotFoundException, IOException{
        InputStream is = this.getClass().getResourceAsStream(fn);
        BufferedReader br = new BufferedReader(new InputStreamReader(is));
//        BufferedReader br = new BufferedReader(new FileReader(fn));
        String line = br.readLine();
        ArrayList<TYPE> typearr = new ArrayList<TYPE>();
        HashSet<Character> separatorList = new HashSet<Character>();
        for(char c : separators)separatorList.add(c);
        
        String str = "";
        for(char c : line.toCharArray()){
            if(separatorList.contains(c)){
                try{
                    Double.parseDouble(str);
                    typearr.add(TYPE.REAL);
                }catch(Exception e){
                    typearr.add(TYPE.NOMINAL);
                }
                str = "";
            }else{
                str += c;
            }
        }
        TYPE[] typeVector = new TYPE[typearr.size()];
        typearr.toArray(typeVector);
        init(fn, typeVector, separators);
    }
    
    /**
     * initialize the data
     * @param fn
     * @param typeVector
     * @param separators
     * @throws FileNotFoundException
     * @throws IOException 
     */
    private void init(String fn, TYPE[] typeVector, char[] separators) throws FileNotFoundException, IOException {
        InputStream is = this.getClass().getResourceAsStream(fn);
        BufferedReader br = new BufferedReader(new InputStreamReader(is));
        ArrayList<ArrayList<String>> rawInstances = new ArrayList<ArrayList<String>>();
        
        HashSet<Character> separatorList = new HashSet<Character>();
        for(char c : separators)separatorList.add(c);
        
        String line = br.readLine();
        while(line != null) {
            String str = "";
            ArrayList<String> ri = new ArrayList<String>();
            for(char c : line.toCharArray()){
                if(separatorList.contains(c)){
                    ri.add(str);
                    str = "";
                }else{
                    str += c;
                }
            }
            ri.add(str);
            rawInstances.add(ri);
            line = br.readLine();
        }
        br.close();
        
        dataVector = new FastVector(typeVector.length);
        int index = 0;
        for(TYPE t : typeVector){
            switch(t){
                case REAL:
                    dataVector.addElement(new Attribute(""+index));
                    break;
                case NOMINAL:
                    HashSet<String> nominals = new HashSet<String>();
                    for(ArrayList<String> ri : rawInstances){
                        nominals.add(ri.get(index));
                    }
                    FastVector fv = new FastVector(nominals.size());
                    for(String s : nominals)fv.addElement(s);
                    dataVector.addElement(new Attribute(index+"", fv));
                    break;
            }
            index++;
        }
        Instances all = new Instances("all", dataVector, rawInstances.size());
        for(ArrayList<String> l : rawInstances){
            double[] ivs = new double[dataVector.size()];
            for(int i = 0; i < dataVector.size(); i++){
                switch(typeVector[i]){
                    case REAL:
                        ivs[i] = Double.parseDouble(l.get(i));
                        break;
                    case NOMINAL:
                        ivs[i] = all.attribute(i).indexOfValue(l.get(i));
                        break;
                }
            }
            all.add(new Instance(1d, ivs));
        }
        all.randomize(new Random(1));
        training = new Instances(all, 0, all.numInstances()/3);
        fitness = new Instances(all, all.numInstances()/3, all.numInstances()/3);
        validation = new Instances(all, 2*all.numInstances()/3, all.numInstances()/3);
        training.setRelationName("training");
        fitness.setRelationName("fitness");
        validation.setRelationName("validation");
    }
    
    public void setClassIndex(int i){
        this.training.setClassIndex(i);
        this.fitness.setClassIndex(i);
        this.validation.setClassIndex(i);
    }
    
    private Instances featureSubSet(Instances instances, int... indexes){
        FastVector nvf = new FastVector(indexes.length);
        for(int i : indexes){
            nvf.addElement(instances.attribute(i));
        }
        Instances ret = new Instances("featureSubSetOf(" + instances.relationName() + ")", nvf, instances.numInstances());
        
        for(int i = 0; i < instances.numInstances(); i++){
            double[] ats = new double[indexes.length];
            for(int j = 0; j < indexes.length; j++)ats[j] = instances.instance(i).value(indexes[j]);
            Instance ni = new Instance(1d, ats);
            ret.add(ni);
        }
        
        ret.setClass(instances.classAttribute());
        
        return ret;
    }
    
    /**
     * create a datawrapper with addition 'noise' indexes
     * @param reals number of realvalued indexes to add
     * @param nominals number of nominal indexes to add, NOT IMPLEMENTED USE 0
     * @return the new dataWrapper
     */
    public DataWrapper getBigger(int reals, int nominals){
        
        DataWrapper ret = new DataWrapper();
        FastVector ndv = new FastVector(this.dataVector.size() + reals + nominals);
        
        int i = 0;
        for(; i < this.dataVector.size(); i++)ndv.addElement(dataVector.elementAt(i));
        
        for(; i < this.dataVector.size() + reals; i++)ndv.addElement(new Attribute(i+""));
        System.out.println(ndv.size());
        Random r = new Random(1);
        ret.dataVector = ndv;
        ret.fitness = getBigger(fitness, ndv, r);
        ret.training = getBigger(training, ndv, r);
        ret.validation = getBigger(validation, ndv, r);
        ret.setClassIndex(training.classIndex());
        return ret;
    }
    
    /**
     * append randomized indexes to a existing instance set
     * @param instances old set of instances
     * @param ndv new featurevector
     * @param r random function to draw from
     * @return new set with random variables
     */
    private Instances getBigger(Instances instances,  FastVector ndv, Random r){
        Instances ret = new Instances("bigger(" + instances.relationName() + ")", ndv, instances.numInstances());
        for(int i = 0; i < instances.numInstances(); i++){
            Instance old = instances.instance(i);
            double[] nats = new double[ndv.size()];
            for(int k = 0; k < ndv.size(); k++){
                if(k < instances.numAttributes())nats[k] = old.value(k);
                else nats[k] = r.nextGaussian();
            }
            ret.add(new Instance(1d, nats));
        }
        return ret;
    }
    
    public static void main(String[] args){
        try {
            DataWrapper dw = new DataWrapper("Yeast.txt", new TYPE[]{TYPE.REAL, TYPE.REAL, TYPE.REAL, TYPE.REAL, TYPE.REAL, TYPE.REAL, TYPE.REAL, TYPE.REAL, TYPE.NOMINAL});
            dw.setClassIndex(8);
            dw = dw.getBigger(10, 0);
            System.out.println(dw.training.toString());
            System.out.println(dw.fitness.toString());
            System.out.println(dw.validation.toString());
        } catch (FileNotFoundException ex) {
            Logger.getLogger(DataWrapper.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(DataWrapper.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
}