/*
 * I OWN ALL THE SYNTAX
 */
package fs.data;

import fs.data.DataWrapper.TYPE;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Random;
import java.util.Set;
import javax.swing.JOptionPane;
import sun.reflect.generics.reflectiveObjects.NotImplementedException;
import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;

/**
 *
 * @author sigved
 */
public class DataWrapperAS implements DataSourceAS {
    private static char[] defSep = new char[]{',',';','|'};
    private FastVector dataVector;
    private Instances training;
    private Instances fitness;
    private Instances validation;
    private Instances all;
    
    public DataWrapperAS(String fn, TYPE[] typeVector, char[] separators) throws FileNotFoundException, IOException{
        init(fn, typeVector, separators);
    }
    public DataWrapperAS(String fn, TYPE[] typeVector) throws FileNotFoundException, IOException{
        init(fn, typeVector, defSep);
    }
    public DataWrapperAS(String fn, char[] separators) throws FileNotFoundException, IOException{
        init(fn, separators);
    }
    public DataWrapperAS(String fn) throws FileNotFoundException, IOException{
        init(fn, defSep);
    }
    public DataWrapperAS(Instances instances){
        this.all = instances;
        this.partition();
        this.setClassIndex(all.classIndex());
    }
    private DataWrapperAS(){
        
    }
    
    /**
     * best effort to construct a type vector, when it's missing
     * @param fn
     * @param separators
     * @throws FileNotFoundException
     * @throws IOException 
     */
    private void init(String fn, char[] separators) throws FileNotFoundException, IOException{
//        InputStream is = this.getClass().getResourceAsStream(fn);
        System.out.println(new File(fn).exists());
        System.out.println(new File(fn).getAbsolutePath());
        InputStream is = new FileInputStream(new File(fn));
        BufferedReader br = new BufferedReader(new InputStreamReader(is));
//        BufferedReader br = new BufferedReader(new FileReader(fn));
        String line = br.readLine();
        ArrayList<TYPE> typearr = new ArrayList<TYPE>();
        HashSet<Character> separatorList = new HashSet<Character>();
        for(char c : separators)separatorList.add(c);
        String str = "";
        for(char c : line.toCharArray()){
            if(separatorList.contains(c)){
                try{
                    Double.parseDouble(str);
                    typearr.add(TYPE.REAL);
                }catch(Exception e){
                    typearr.add(TYPE.NOMINAL);
                }
                str = "";
            }else{
                str += c;
            }
        }
        if(str.length() > 0){
            try{
                Double.parseDouble(str);
                typearr.add(TYPE.REAL);
            }catch(Exception e){
                typearr.add(TYPE.NOMINAL);
            }
        }
        TYPE[] typeVector = new TYPE[typearr.size()];
        typearr.toArray(typeVector);
        System.out.println("made typevector");
        init(fn, typeVector, separators);
    }
    
    /**
     * initialize the data
     * @param fn
     * @param typeVector
     * @param separators
     * @throws FileNotFoundException
     * @throws IOException 
     */
    private void init(String fn, TYPE[] typeVector, char[] separators) throws FileNotFoundException, IOException {
        InputStream is = new FileInputStream(new File(fn));
        BufferedReader br = new BufferedReader(new InputStreamReader(is));
        ArrayList<ArrayList<String>> rawInstances = new ArrayList<ArrayList<String>>();
        
        HashSet<Character> separatorList = new HashSet<Character>();
        for(char c : separators)separatorList.add(c);
        
        String line = br.readLine();
        while(line != null) {
            String str = "";
            ArrayList<String> ri = new ArrayList<String>();
            for(char c : line.toCharArray()){
                if(separatorList.contains(c)){
                    ri.add(str);
                    str = "";
                }else{
                    str += c;
                }
            }
            ri.add(str);
            rawInstances.add(ri);
            line = br.readLine();
        }
        br.close();
        
        
        dataVector = new FastVector(typeVector.length);
        int index = 0;
        for(TYPE t : typeVector){
            switch(t){
                case REAL:
                    dataVector.addElement(new Attribute(""+index));
                    break;
                case NOMINAL:
                    HashSet<String> nominals = new HashSet<String>();
                    for(ArrayList<String> ri : rawInstances){
                        nominals.add(ri.get(index));
                    }
                    FastVector fv = new FastVector(nominals.size());
                    for(String s : nominals)fv.addElement(s);
                    dataVector.addElement(new Attribute(index+"", fv));
                    break;
            }
            index++;
        }
        all = new Instances("all", dataVector, rawInstances.size());
        for(ArrayList<String> l : rawInstances){
            double[] ivs = new double[dataVector.size()];
            for(int i = 0; i < dataVector.size(); i++){
                switch(typeVector[i]){
                    case REAL:
                        ivs[i] = Double.parseDouble(l.get(i));
                        break;
                    case NOMINAL:
                        ivs[i] = all.attribute(i).indexOfValue(l.get(i));
                        break;
                }
            }
            all.add(new Instance(1d, ivs));
        }
        all.randomize(new Random(1));
        partition();
    }
    
    public void setClassIndex(int i){
        try{
            if(i < 0 || i >= this.all.numAttributes())
                throw new Exception("setClass index to non existant index");
        }catch(Exception e){
            e.printStackTrace();
            JOptionPane.showMessageDialog(null, e.toString());
        }
        this.all.setClassIndex(i);
        this.training.setClassIndex(i);
        this.fitness.setClassIndex(i);
        this.validation.setClassIndex(i);
    }

    @Override
    public synchronized DataSourceAS getFeatureSubSet(Set<Integer> is) {
        ArrayList<Integer> indexes = new ArrayList<Integer>(is);
        DataWrapperAS ret = new DataWrapperAS();
        
        ret.dataVector = new FastVector(indexes.size());
        for(int i : indexes){
            ret.dataVector.addElement(all.attribute(i).copy(all.attribute(i).name()));
        }
        
        ret.all = new Instances("featureSubSetOf(" + this.all.relationName() + ")",
                ret.dataVector, this.all.numInstances());
        
        for(int i = 0; i < this.all.numInstances(); i++){
            double[] ats = new double[indexes.size()];
            for(int j = 0; j < indexes.size(); j++)ats[j] = this.all.instance(i).value(indexes.get(j));
            Instance ni = new Instance(1d, ats);
            ret.all.add(ni);
        }
        
        ret.partition();
        
        ret.setClassIndex(indexes.indexOf(this.all.classIndex()));
        
        return ret;
    }

    @Override
    public Instances getTraining() {
        return training;
    }

    @Override
    public Instances getFitness() {
        return fitness;
    }

    @Override
    public Instances getValidation() {
        return validation;
    }
        @Override
    public Instances getAll() {
        return all;
    }

    
    public Instances getFitnessAndTraining() {
        return new Instances(all, 0, (all.numInstances()/3)*2);
    }

    private void partition() {
        training = new Instances(all, 0, all.numInstances()/3);
        fitness = new Instances(all, all.numInstances()/3, all.numInstances()/3);
        validation = new Instances(all, 2*all.numInstances()/3, all.numInstances()/3);
        training.setRelationName("training");
        fitness.setRelationName("fitness");
        validation.setRelationName("validation");
    }

    @Override
    public Integer getClassIndex() {
        return all.classIndex();
    }
    
        
    /**
     * create a datawrapper with addition 'noise' indexes
     * @param reals number of realvalued indexes to add
     * @param nominals number of nominal indexes to add, NOT IMPLEMENTED USE 0
     * @return the new dataWrapper
     */
    public DataWrapperAS getBigger(int reals, int nominals){
        
        DataWrapperAS ret = new DataWrapperAS();
        FastVector ndv = new FastVector(this.dataVector.size() + reals + nominals);
        
        int i = 0;
        for(; i < this.dataVector.size(); i++)ndv.addElement(dataVector.elementAt(i));
        
        for(; i < this.dataVector.size() + reals; i++)ndv.addElement(new Attribute(i+""));
        Random r = new Random(1);
        ret.dataVector = ndv;
        ret.all = getBigger(all, ndv, r);
        ret.partition();
        ret.setClassIndex(this.getClassIndex());
        return ret;
    }
    
    /**
     * append randomized indexes to a existing instance set
     * @param instances old set of instances
     * @param ndv new featurevector
     * @param r random function to draw from
     * @return new set with random variables
     */
    private Instances getBigger(Instances instances,  FastVector ndv, Random r){
        Instances ret = new Instances("bigger(" + instances.relationName() + ")", ndv, instances.numInstances());
        for(int i = 0; i < instances.numInstances(); i++){
            Instance old = instances.instance(i);
            double[] nats = new double[ndv.size()];
            for(int k = 0; k < ndv.size(); k++){
                if(k < instances.numAttributes())nats[k] = old.value(k);
                else nats[k] = r.nextGaussian();
            }
            ret.add(new Instance(1d, nats));
        }
        return ret;
    }

    public int numIndexes() {
        return all.numAttributes();
    }

    public TYPE[] getTYPEArray() throws Exception {
        TYPE[] ret = new TYPE[dataVector.size()];
        for(int i = 0; i < ret.length; i++){
            if(((Attribute)dataVector.elementAt(i)).isNominal())ret[i] = TYPE.NOMINAL;
            else if(((Attribute)dataVector.elementAt(i)).isNumeric())ret[i] = TYPE.REAL;
            else throw new Exception("somethings wrong in getTYPEArray()");
        }
        return ret;
    }



}
